#!/usr/bin/env python

import gtk

def Run(*args):
  gtk.mainloop()

def Quit(*args):
  gtk.mainquit()

def runqueue():
  while gtk.events_pending():
    gtk.mainiteration()

class ButtonChoiceWindow:
  """Simple convenience object for multiple-choice user queries"""

  def __init__(self):
    pass

  def __call__(self, title, text, buttons, default=0, long=0,
               transient=None):
    self._clicked = None

    win = gtk.Window()
    if transient:
      win.set_transient_for(transient)
    win.connect("destroy", gtk.mainquit)
    win.set_position(gtk.WIN_POS_MOUSE)
    win.set_modal(1)
    win.set_title(title)
    win.set_border_width(5)

    v = gtk.VBox()

    #if len(text) < 300:
    if not long:

      l = gtk.Label(text)
      l.set_line_wrap(1)
      l.show()
      v.add(l)

    else:  #if len(text) > 300:

      #lbox = gtk.HBox()
      tbox = gtk.ScrolledWindow()
      l = gtk.TextView()
      l.set_editable(0)
      l.set_cursor_visible(0)
      l.set_wrap_mode(gtk.WRAP_WORD)
      #l = gtk.Text(None, None)
      #l.set_word_wrap(1)
      #l.set_line_wrap(1)
      #scroll = gtk.VScrollbar(l.get_vadjustment())
      tbox.add(l)
      #lbox.pack_start(l, 1, 1)
      #lbox.pack_start(scroll, 0, 0)
      #l.insert(None, None, None, text)
      tb = l.get_buffer()
      tb.set_text(text)
      #l.set_usize(400, 300)
      v.pack_start(tbox, 1, 1)


    box = gtk.HButtonBox()
    box.set_border_width(10)


    buttonlist = []
    i = 0
    for item in buttons:
      if type(item) == type((1,2)):
        label, value = item
      else:
        label = value = item


      b = gtk.Button(label)
      buttonlist.append([label, value, b])
      b.connect("clicked", self._cb)

      if default >= 0  and  i == default:
        b.set_flags(gtk.CAN_DEFAULT)
        b.set_flags(gtk.HAS_DEFAULT)
        box.add(b)
      else:
        f = gtk.Frame()
        f.set_border_width(2)
        f.set_shadow_type(gtk.SHADOW_NONE)
        f.add(b)
        box.add(f)

      i = i + 1

    box.show()
    v.pack_start(box, 0, 0)

    v.show()
    win.add(v)

    win.show_all()
    gtk.mainloop()

    retval = None

    for item in buttonlist:
      label, value, b = item
      if b == self._clicked:
        retval = value

    win.hide()
    win.unrealize()
    del win

    return retval

  def _cb(self, button):
    self._clicked = button
    gtk.mainquit()

def EntryWindow(title, text, entry, buttons = ["Ok", "Cancel"], hidden=0, 
                transient=None):
  win = Window(title)
  win.set_modal(1)
  win.w.set_position(gtk.WIN_POS_MOUSE)
  if transient:
    win.w.set_transient_for(transient)

  l = Label(text)
  l.set_padding(5)
  win.add(l)

  if type(entry) == type("Hi"):
    e = Entry("", entry, hidden=hidden)
  elif type(entry) == type((1,2)):
    label, text = entry
    e = Entry(text, label, hidden=hidden)
  e.set_padding(5)
  win.add(e)

  bbar = ButtonBar(buttons)
  bbar.set_padding(5)
  win.add(bbar)
  

  e.w.grab_focus()
  win.show()
  Run()
  win.hide()
  win.w.unrealize()

  retval = None

  entered = e.get_value()
  clicked = bbar.get_value()

  retval = (clicked, entered)

  return retval

class Widget:
  """Widget class...
  
  Conventions:
   - classes derived from Widget should name their primary gtk.+
     widget "self.w".  This allows callbacks to work.
   - classes derived from Widget should call "self._init()" in
     their __init__()."""
  def __init__(self):
    self._init()

  def _init(self):
    self._cb = {}
    self._cb_func = {
      "pressed" : self._pressed_cb,
      "released" : self._released_cb,
      "clicked" : self._clicked_cb,
      "toggled" : self._toggled_cb,
      "enter" : self._enter_cb,
      "leave" : self._leave_cb,
      "destroy" : self._destroy_cb,
      "" : None
      }


  def _pressed_cb(self, *args):
    self._callback("pressed")

  def _released_cb(self, *args):
    self._callback("released")

  def _clicked_cb(self, *args):
    self._callback("clicked")

  def _toggled_cb(self, *args):
    global _object_table
    (widget, ) = args
    value = widget.get_active()
    self._callback("toggled", value)

  def _enter_cb(self, *args):
    self._callback("enter")

  def _leave_cb(self, *args):
    self._callback("leave")

  def _destroy_cb(self, *args):
    self._callback("destroy")

  def _callback(self, event, *args):
    if self._cb.has_key(event):
      parameters = (self, event) + args
      #print "Calling %s cb...  Args: %s" % (event, repr(parameters))
      apply(self._cb[event], parameters)
      #print "Calling %s cb... done.\n" % event
  
  def set_cb(self, event, cb):
    if self._cb_func.has_key(event)  and  hasattr(self, "w"):
      self._cb[event] = cb
      self.w.connect(event, self._cb_func[event])
    else:
      raise ValueError, "Invalid event type %s" % event

  def run(self):
    Run()

  def show(self):
    if hasattr(self, "w"):
      self.w.show()
    
  def hide(self):
    if hasattr(self, "w"):
      self.w.hide()

  def set_padding(self, xpad, ypad=None):
    if ypad == None:
      ypad = xpad
    if hasattr(self, "w"):
      if hasattr(self.w, "set_padding"):
        self.w.set_padding(xpad, ypad)
      elif hasattr(self.w, "set_border_width"):
        self.w.set_border_width(xpad)

class Label(Widget):
  def __init__(self, text=""):
    self._init()
    self.w = gtk.Label(text)
    self.w.set_line_wrap(1)

  def set_text(self, text):
    self.w.set_text(text)

class Entry(Widget):
  def __init__(self, text=None, label=None, hidden=0):
    self._init()
    self.w = gtk.HBox()
    self.e = gtk.Entry()
    self.l = None
    if label:
      self.l = gtk.Label(label)
      self.w.add(self.l)
    if text:
      self.set_text(text)
    self.w.add(self.e)
    if hidden:
      self._set_visible(0)

  def set_text(self, text):
    self.e.set_text(text)

  def _set_visible(self, visible):
    self.e.set_visibility(visible)

  def get_value(self):
    return self.e.get_text()

class Button(Widget):
  def __init__(self, text=None):
    self._init()
    self.w = gtk.Button()
    if text:
      self.set_text(text)

  def set_text(self, text):
    if hasattr(self, "l"):
      self.l.set_text(text)
    else:
      self.l = gtk.Label()
      self.l.set_line_wrap(1)
      self.l.set_padding(5, 3)
      self.l.set_text(text)
      self.w.add(self.l)

class ButtonBar(Widget):
  def __init__(self, buttons=[]):
    self._init()
    self.w = gtk.HButtonBox()
    #self.w.set_layout_default(gtk.BUTTONBOX_SPREAD)
    self.w.set_layout(gtk.BUTTONBOX_SPREAD)
    self.buttons = []
    self._clicked = None
    for button in buttons:
      if type(button) == type((1,2)):
        text, value = button
      else:
        text = value = button
      b = Button(text)
      b.set_cb("clicked", self._default_cb)
      self.buttons.append((b, value))
      self.w.add(b.w)

  def set_cb(self, which, cb):
    try:
      (button, value) = self.buttons[which]
    except:
      raise IndexError, "Button out of range"
    button.set_cb("clicked", cb)

  def _default_cb(self, object, event):
    for item in self.buttons:
      (button, value) = item
      if button == object:
        self._clicked = value
        #print "Buttonbar clicked: %s" % value
        Quit()

  def get_value(self, result=None):
    return self._clicked

class Window(Widget):
  def __init__(self, title="", modal=0):
    self._init()
    self.w = gtk.Window()
    self.w.set_position(gtk.WIN_POS_CENTER)
    self.w.connect("destroy", gtk.mainquit)
    self.w.set_title(title)
    self.w.set_modal(modal)
    self.box = gtk.Table(99,99)
    self.w.add(self.box)
    self._current = 0
    #self.w.set_border_width(5)

  def set_title(self, title):
    self.w.set_title(title)

  def set_modal(self, modal):
    self.w.set_modal(modal)

  def show(self):
    self.w.show_all()

  def hide(self):
    self.w.hide()

  def add(self, object, x=-1, y=-1, expand=0, gravity=None, padding=1):
    if object  and  hasattr(object, "w"):
      #self.box.add(object.w)
      if x>=0  and  y>=0:
        if expand:
          self.box.attach(object.w, x, x+1, y, y+1)
        else:
          self.box.attach(object.w, x, x+1, y, y+1, xoptions=0, yoptions=0)
      else:
        self.box.attach(object.w, 0, 1, self._current, self._current+1)
        #                xoptions=0, yoptions=0)
        self._current = self._current + 1

  def run(self, *args):
    self.show()
    Run()

  def quit(self, *args):
    Quit()

  def __del__(self):
    self.w.hide()
    self.w.unrealize()
    del self.w

def run(args):
  a = ButtonChoiceWindow()("Error", "You have no idea what you're doing, do you?",
		     [("Shut up!", "angry"), ("No...", "defeated")], 1)
  print "Result: %s" % (a)

if __name__ == "__main__":
  import sys
  run(sys.argv[1:])
   
